//******************************************************************************
//
//
//   Rele atuador com sensor ultrasonico   v.2
//
//
//   Autor:Claudio Larios
//
//   Data: 27/06/2014
//
//   Aciona rele quando objeto se aproxima do sensor
//   Temporizado em 5 minutos usando timer 0
//   Potenciometro de ajuste da distancia de sensibilidade
//
//   Montagem didtica.
//   Utiliza PIC12f675 e sensores ultrasonico HC-SR-04.
//   Parte integrante do blog Larios.Tecnologia.ws
//******************************************************************************

#include <12F675.h>//arquivos de cabealho
#device adc=8 // 8 bits
#use delay(clock=4000000) //clock interno de 4 mhz
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, MCLR, NOPUT, BROWNOUT //fuses

//==============================================================================
// Tempo que a sada fica ligada aps acionamento:
//

#define tempo_longo   //opo para minutos acionados

//
// Descomente para tempo de 5 minutos ligado e comente para tempo de 10 segundos
//==============================================================================

// definies de registradores
#byte tmr1h   = 0x0f
#byte tmr1l   = 0x0e
#byte t1con   = 0x10
#byte pir1    = 0x0c
#byte gpio    = 0x05
#byte tris    = 0x85
#byte intcon  =0x0b
#byte tmr0    =0x01

//bits
#bit  on_tmr1 = 0x10.0
#bit  over    = 0x0c.0
#bit  echo    = 0x05.4
#bit  trig    = 0x05.5
#bit  saida   = 0x05.1 //saida do rele
#bit  t0if    = 0x0b.2 //flag tmr0
//variavis globais
int val_pot;
int16 tempo_lig;

//constantes
#ifdef tempo_longo
const byte  tempo  = 5 ; //valor do tempo ligado (em minutos)
const long byte carga_tempo =tempo * 60000000 / 65536;  
#else
const byte  tempo  = 10 ; //valor do tempo ligado (em segundos)
const long byte carga_tempo =tempo * 1000000 / 65536;  

#endif

//==============================================================================
//     Programa principal
//==============================================================================


void main()
{

   setup_adc_ports(sAN2 |VSS_VDD); // acerta pino 5 para ler tenso com adc
   setup_adc(ADC_CLOCK_DIV_8);//frequencia de trabalho do conversor adc
   set_adc_channel(2);//escolhe o canal 2 pino 5
   delay_us(10);//delay
   setup_counters(RTCC_INTERNAL,RTCC_DIV_256);//configura timer 0
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores para liberar ports
   setup_vref(FALSE);//desliga vref
   tris=0b011101;//ajusta registrador de direo da port
   gpio=0;//desliga tudo
   tmr0=0;
   t0if=0;
//==============================================================================
//     loop principal
//==============================================================================
   WHILE(TRUE){
   on_tmr1=0; //reseta timer 1
   tmr1h=0;
   tmr1l=0;
   over=0; //apaga flag de over flow do timer 1
//==============================================================================
//     Obtem valor da tenso no potenciometro
//==============================================================================

   val_pot = read_adc()/4;// le potenciometro do 'alcance'

//==============================================================================
//    Obtm a distncia do objeto (usando timer 1)
//==============================================================================
   trig=1; //seta
   delay_us(13);
   trig=0;
   while(!echo);//aguarda ficar '1'
   on_tmr1=1;//liga timer
   while(echo);//aguarda ficar '0'
   on_tmr1=0; //desliga timer 1
   

   
//==============================================================================
//    Compara com valor obtido no potenciometro
//==============================================================================

   if((tmr1h<val_pot)&& (!over)){
    saida=1;
    tempo_lig=carga_tempo;
    tmr0=0;
    t0if=0;
    }//liga saida
   delay_ms(10);//delay
   
//==============================================================================
//    Rotina de desligamento por tempo (usando timer 0)
//==============================================================================   
   if(t0if&&saida) {
    t0if=0;
    if(!--tempo_lig) saida=0;
    }   
  }//while
}//main
